/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:          ctsim.cpp
**   Purpose:       Top-level routines for CTSim program
**   Programmer:    Kevin Rosenberg
**   Date Started:  July 2000
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: ctsim.h,v 1.29 2001/01/20 08:10:33 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#ifndef __CTSIMH__
#define __CTSIMH__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef HAVE_DMALLOC
#include <dmalloc.h>
#endif

#ifdef MSVC
#define CTSIM_MDI 1
#endif

#if defined(CTSIM_MDI) && !wxUSE_MDI_ARCHITECTURE
#error You must set wxUSE_MDI_ARCHITECTURE to 1 in setup.h!
#endif
#ifdef CTSIM_MDI
#include "wx/docmdi.h"
#endif

class wxMenu;
class wxDocument;

class ImageFileDocument;

#include <vector>

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/docview.h"
#include "wx/textctrl.h"
#include "wx/menu.h"
#include "wx/help.h"
#include "wx/html/helpctrl.h"
#include "dlgezplot.h"


#if defined(__WXMSW__)
#define CTSIM_WINHELP   1
#endif

// Define a new frame for main window
#if CTSIM_MDI
class MainFrame: public wxMDIParentFrame
#else
class MainFrame: public wxDocParentFrame
#endif
{
  DECLARE_CLASS(MainFrame)
private:
  wxTextCtrl* m_pLog;
  wxMenu* m_pWindowMenu;
  
  enum { MAX_WINDOW_MENUITEMS = 20 };
  wxMenuItem* m_apWindowMenuItems[MAX_WINDOW_MENUITEMS];
  wxDocument* m_apWindowMenuData[MAX_WINDOW_MENUITEMS];
  
  int m_iDefaultPhantomID;
  int m_iDefaultFilterID;
  int m_iDefaultFilterDomainID;
  unsigned int m_iDefaultFilterXSize;    
  unsigned int m_iDefaultFilterYSize;
  double m_dDefaultFilterParam;
  double m_dDefaultFilterBandwidth;
  double m_dDefaultFilterInputScale;
  double m_dDefaultFilterOutputScale;
  
#if CTSIM_WINHELP
  wxWinHelpController      m_winHelp;
#endif
  wxHtmlHelpController     m_htmlHelp;
  
public:
  MainFrame(wxDocManager *manager, wxFrame *frame, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, const long type);

  void OnMRUFile (wxCommandEvent& event);

  void OnSize (wxSizeEvent& event);

#if CTSIM_WINHELP
  wxWinHelpController&   getWinHelpController()
  {return m_winHelp; }
#endif
  wxHtmlHelpController&  getHtmlHelpController()
  { return m_htmlHelp; }

  void showHelp (int commandID);

  void OnAbout (wxCommandEvent& event);
  void OnHelpTopics (wxCommandEvent& event);
  void OnHelpContents (wxCommandEvent& event);
  void OnCreatePhantom (wxCommandEvent& event);
  
#if defined(CTSIM_WINHELP) && (defined(DEBUG) || defined(_DEBUG))
  void OnHelpSecondary (wxCommandEvent& event);
#endif

  void OnCreateFilter (wxCommandEvent& event);
  void OnExit (wxCommandEvent& event);
  
  void OnUpdateUI (wxUpdateUIEvent& event);
  
  wxTextCtrl* getLog() 
  { return m_pLog; }

  void OnWindowMenu0 (wxCommandEvent& event);
  void OnWindowMenu1 (wxCommandEvent& event);
  void OnWindowMenu2 (wxCommandEvent& event);
  void OnWindowMenu3 (wxCommandEvent& event);
  void OnWindowMenu4 (wxCommandEvent& event);
  void OnWindowMenu5 (wxCommandEvent& event);
  void OnWindowMenu6 (wxCommandEvent& event);
  void OnWindowMenu7 (wxCommandEvent& event);
  void OnWindowMenu8 (wxCommandEvent& event);
  void OnWindowMenu9 (wxCommandEvent& event);
  void OnWindowMenu10 (wxCommandEvent& event);
  void OnWindowMenu11 (wxCommandEvent& event);
  void OnWindowMenu12 (wxCommandEvent& event);
  void OnWindowMenu13 (wxCommandEvent& event);
  void OnWindowMenu14 (wxCommandEvent& event);
  void OnWindowMenu15 (wxCommandEvent& event);
  void OnWindowMenu16 (wxCommandEvent& event);
  void OnWindowMenu17 (wxCommandEvent& event);
  void OnWindowMenu18 (wxCommandEvent& event);
  void OnWindowMenu19 (wxCommandEvent& event);
  
  void DoWindowMenu (int iMenuPosition, wxCommandEvent& event);
  
  DECLARE_EVENT_TABLE()
};


class wxDocManager;
class CTSimApp: public wxApp
{
public:
  CTSimApp();
  bool OnInit();
  int OnExit();
  MainFrame* getMainFrame() const
  { return m_pFrame; }

  wxTextCtrl* getLog()
  { return m_pFrame->getLog(); }

  wxDocManager* getDocManager() 
  { return m_docManager; }
  
  wxString getUntitledFilename();
  
  EZPlotDialog* makeEZPlotDialog()
  { return new EZPlotDialog (m_pFrame); }
  
  void getCompatibleImages (const ImageFileDocument* pIFDoc, std::vector<ImageFileDocument*>& vecIF);
  
  bool getSetModifyNewDocs() const
  { return true; }
  
  void setIconForFrame (wxFrame* pFrame);

private:
  wxDocManager* m_docManager;
  MainFrame* m_pFrame;
  
  void usage (const char* program);
  
  static struct option ctsimOptions[];
  
  enum { O_HELP, O_VERSION };
};

DECLARE_APP(CTSimApp)

extern class CTSimApp* theApp;

enum {
    MAINMENU_WINDOW_BASE = 500,
    MAINMENU_HELP_ABOUT = 600,
    MAINMENU_HELP_CONTENTS,
    MAINMENU_HELP_TOPICS,
    MAINMENU_HELP_SECONDARY,
    MAINMENU_FILE_CREATE_PHANTOM,
    
    MAINMENU_FILE_CREATE_FILTER,
    MAINMENU_FILE_EXIT,
    
    PJMENU_FILE_PROPERTIES,
    PJMENU_RECONSTRUCT_FBP,
    PJMENU_RECONSTRUCT_FOURIER,
    PJMENU_CONVERT_POLAR,
    PJMENU_CONVERT_FFT_POLAR,
    
    IFMENU_FILE_EXPORT,
    IFMENU_FILE_PROPERTIES,
    
    IFMENU_PLOT_ROW,
    IFMENU_PLOT_COL,
    IFMENU_PLOT_FFT_ROW,
    IFMENU_PLOT_FFT_COL,
    IFMENU_PLOT_HISTOGRAM,
    
    IFMENU_VIEW_SCALE_AUTO,
    IFMENU_VIEW_SCALE_MINMAX,
    IFMENU_VIEW_SCALE_FULL,
    
    IFMENU_COMPARE_IMAGES,
    IFMENU_COMPARE_ROW,
    IFMENU_COMPARE_COL,
    IFMENU_IMAGE_SCALESIZE,
    IFMENU_IMAGE_ADD,
    IFMENU_IMAGE_SUBTRACT,
    IFMENU_IMAGE_MULTIPLY,
    IFMENU_IMAGE_DIVIDE,
    
    IFMENU_FILTER_INVERTVALUES,
    IFMENU_FILTER_SQRT,
    IFMENU_FILTER_SQUARE,
    IFMENU_FILTER_LOG,
    IFMENU_FILTER_EXP,
    IFMENU_FILTER_FOURIER,
    IFMENU_FILTER_INVERSE_FOURIER,
    IFMENU_FILTER_FFT,
    IFMENU_FILTER_IFFT,
    IFMENU_FILTER_FFT_ROWS,
    IFMENU_FILTER_FFT_COLS,
    IFMENU_FILTER_IFFT_ROWS,
    IFMENU_FILTER_IFFT_COLS,
    IFMENU_FILTER_MAGNITUDE,
    IFMENU_FILTER_PHASE,
    IFMENU_FILTER_SHUFFLENATURALTOFOURIERORDER,
    IFMENU_FILTER_SHUFFLEFOURIERTONATURALORDER,
    
    PHMMENU_FILE_PROPERTIES,
    PHMMENU_PROCESS_RASTERIZE,
    PHMMENU_PROCESS_PROJECTIONS,
    
    PLOTMENU_VIEW_SCALE_MINMAX,
    PLOTMENU_VIEW_SCALE_AUTO,
    PLOTMENU_VIEW_SCALE_FULL,    
};

#endif
