/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:          view.h
**   Purpose:       Header file for View & Canvas routines of CTSim program
**   Programmer:    Kevin Rosenberg
**   Date Started:  July 2000
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: views.h,v 1.27 2001/01/20 08:10:33 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#ifdef __GNUG__
// #pragma interface
#endif

#ifndef __VIEWSH__
#define __VIEWSH__

#include "wx/wx.h"
#include "docs.h"
#include "imagefile.h"

class ImageFileCanvas;
class ImageFileView : public wxView
{
  DECLARE_DYNAMIC_CLASS(ImageFileView)
    
    wxMemoryDC m_memoryDC;
  wxBitmap m_bitmap;
  
private:
  ImageFileCanvas *CreateCanvas(wxView *view, wxFrame *parent);
  wxFrame *CreateChildFrame(wxDocument *doc, wxView *view);
  
  ImageFileCanvas *m_canvas;
  wxFrame *m_frame;
  bool m_bMinSpecified;
  bool m_bMaxSpecified;
  double m_dMinPixel;
  double m_dMaxPixel;
  double m_dAutoScaleFactor;
  
  int m_iDefaultExportFormatID;

  wxFrame* getFrameForChild() 
#if CTSIM_MDI
  { return theApp->getMainFrame(); }
#else
  { return m_frame; }
#endif

public:
  ImageFileView(void);
  virtual ~ImageFileView(void);
  
  bool OnCreate(wxDocument *doc, long flags);
  void OnDraw(wxDC* dc);
  void OnUpdate(wxView *sender, wxObject *hint = NULL);
  bool OnClose (bool deleteWindow = true);
  
  void OnScaleSize (wxCommandEvent& event);
  void OnExport (wxCommandEvent& event);
  
  void OnProperties (wxCommandEvent& event);
  void OnCompare (wxCommandEvent& event);
  
  void OnInvertValues (wxCommandEvent& event);
  void OnSquare (wxCommandEvent& event);
  void OnSquareRoot (wxCommandEvent& event);
  void OnLog (wxCommandEvent& event);
  void OnExp (wxCommandEvent& event);
  void OnAdd (wxCommandEvent& event);
  void OnSubtract (wxCommandEvent& event);
  void OnMultiply (wxCommandEvent& event);
  void OnDivide (wxCommandEvent& event);
  void OnFourier (wxCommandEvent& event);
  void OnInverseFourier (wxCommandEvent& event);
  void OnShuffleNaturalToFourierOrder (wxCommandEvent& event);
  void OnShuffleFourierToNaturalOrder (wxCommandEvent& event);
  
#ifdef HAVE_FFT
  void OnFFT (wxCommandEvent& event);
  void OnIFFT (wxCommandEvent& event);
  void OnFFTRows (wxCommandEvent& event);
  void OnIFFTRows (wxCommandEvent& event);
  void OnFFTCols (wxCommandEvent& event);
  void OnIFFTCols (wxCommandEvent& event);
#endif
  
  void OnMagnitude (wxCommandEvent& event);
  void OnPhase (wxCommandEvent& event);
  
  void OnScaleAuto (wxCommandEvent& event);
  void OnScaleMinMax (wxCommandEvent& event);
  void OnScaleFull (wxCommandEvent& event);
  void OnPlotRow (wxCommandEvent& event);
  void OnPlotCol (wxCommandEvent& event);
#if HAVE_FFT
  void OnPlotFFTRow (wxCommandEvent& event);
  void OnPlotFFTCol (wxCommandEvent& event);
#endif
  void OnPlotHistogram (wxCommandEvent& event);
  void OnCompareRow (wxCommandEvent& event);
  void OnCompareCol (wxCommandEvent& event);
  
  wxFrame* getFrame()
  { return m_frame; }
  
  ImageFileDocument* GetDocument(void) 
  { return dynamic_cast<ImageFileDocument*>(wxView::GetDocument()); }
  
  DECLARE_EVENT_TABLE()
};

class ImageFileCanvas: public wxScrolledWindow
{
private:
  
  int m_xCursor;
  int m_yCursor;
  ImageFileView* m_pView;
  
public:
  ImageFileCanvas (ImageFileView* v, wxFrame *frame, const wxPoint& pos, const wxSize& size, const long style);
  virtual ~ImageFileCanvas()
  {}

  virtual void OnDraw(wxDC& dc);
  void OnMouseEvent(wxMouseEvent& event);
  void DrawRubberBandCursor (wxDC& dc, int x, int y);
  bool GetCurrentCursor (int& x, int& y);

  virtual wxSize GetBestSize() const;

  DECLARE_EVENT_TABLE()
};


class ProjectionFileCanvas;
class ProjectionFileView : public wxView
{
  DECLARE_DYNAMIC_CLASS(ProjectionFileView)
    
    wxMemoryDC m_memoryDC;
  wxBitmap m_bitmap;
  
private:
  ProjectionFileCanvas *CreateCanvas(wxView *view, wxFrame *parent);
  wxFrame *CreateChildFrame(wxDocument *doc, wxView *view);
  
  ProjectionFileCanvas *m_canvas;
  wxFrame *m_frame;
  
  int m_iDefaultNX;
  int m_iDefaultNY;
  int m_iDefaultFilter;
  int m_iDefaultFilterMethod;
  double m_dDefaultFilterParam;
  int m_iDefaultFilterGeneration;
  int m_iDefaultZeropad;
  int m_iDefaultInterpolation;
  int m_iDefaultInterpParam;
  int m_iDefaultBackprojector;
  int m_iDefaultTrace;
  
  int m_iDefaultPolarNX;
  int m_iDefaultPolarNY;
  int m_iDefaultPolarInterpolation;
  int m_iDefaultPolarZeropad;

  wxFrame* getFrameForChild() 
#if CTSIM_MDI
  { return theApp->getMainFrame(); }
#else
  { return m_frame; }
#endif

public:
  ProjectionFileView(void);
  virtual ~ProjectionFileView(void);
  
  bool OnCreate(wxDocument *doc, long flags);
  void OnDraw(wxDC* dc);
  void OnUpdate(wxView *sender, wxObject *hint = NULL);
  bool OnClose (bool deleteWindow = true);
  void OnProperties (wxCommandEvent& event);
  void OnReconstructFBP (wxCommandEvent& event);
  void OnReconstructFourier (wxCommandEvent& event);
  void OnConvertPolar (wxCommandEvent& event);
  void OnConvertFFTPolar (wxCommandEvent& event);
  
  wxFrame* getFrame ()
  { return m_frame; }
  
  ProjectionFileDocument* GetDocument(void) 
  { return dynamic_cast<ProjectionFileDocument*>(wxView::GetDocument()); }
  DECLARE_EVENT_TABLE()
};

class ProjectionFileCanvas: public wxScrolledWindow
{
private:
  ProjectionFileView* m_pView;

public:  
  ProjectionFileCanvas (ProjectionFileView* v, wxFrame *frame, const wxPoint& pos, const wxSize& size, const long style);
  virtual ~ProjectionFileCanvas() 
  {}

  virtual wxSize GetBestSize() const;

  virtual void OnDraw(wxDC& dc);
};


class PhantomCanvas;
class PhantomView : public wxView
{
  DECLARE_DYNAMIC_CLASS(PhantomView)
    
private:
  PhantomCanvas *CreateCanvas(wxView *view, wxFrame *parent);
  wxFrame *CreateChildFrame(wxDocument *doc, wxView *view);
  
  PhantomCanvas *m_canvas;
  wxFrame *m_frame;
  
  int m_iDefaultNDet;
  int m_iDefaultNView;
  int m_iDefaultNSample;
  int m_iDefaultGeometry;
  int m_iDefaultTrace;
  double m_dDefaultRotation;
  double m_dDefaultFocalLength;
  double m_dDefaultFieldOfView;
  
  int m_iDefaultRasterNX;
  int m_iDefaultRasterNY;
  int m_iDefaultRasterNSamples;

  wxFrame* getFrameForChild() 
#if CTSIM_MDI
  { return theApp->getMainFrame(); }
#else
  { return m_frame; }
#endif

public:
  PhantomView(void);
  virtual ~PhantomView(void);
  
  bool OnCreate(wxDocument *doc, long flags);
  void OnUpdate(wxView *sender, wxObject *hint = NULL);
  bool OnClose (bool deleteWindow = true);
  void OnDraw(wxDC* dc);
  void OnProperties (wxCommandEvent& event);
  void OnRasterize (wxCommandEvent& event);
  void OnProjections (wxCommandEvent& event);
  
  PhantomDocument* GetDocument(void) 
  { return dynamic_cast<PhantomDocument*>(wxView::GetDocument()); }
  
  DECLARE_EVENT_TABLE()
};

class PhantomCanvas: public wxScrolledWindow
{
private:
  PhantomView* m_pView;

public:  
  PhantomCanvas (PhantomView* v, wxFrame *frame, const wxPoint& pos, const wxSize& size, const long style);
  virtual ~PhantomCanvas()
  {}

  virtual void OnDraw(wxDC& dc);
};

class PlotFileCanvas;
class PlotFileView : public wxView
{
  DECLARE_DYNAMIC_CLASS(PlotFileView)
    
private:
  PlotFileCanvas *CreateCanvas(wxView *view, wxFrame *parent);
  wxFrame *CreateChildFrame(wxDocument *doc, wxView *view);
  
  PlotFileCanvas *m_canvas;
  wxFrame *m_frame;
  EZPlot* m_pEZPlot;
  
  bool m_bMinSpecified;
  bool m_bMaxSpecified;
  double m_dMinPixel;
  double m_dMaxPixel;
  double m_dAutoScaleFactor;
  
  wxFrame* getFrameForChild() 
#if CTSIM_MDI
  { return theApp->getMainFrame(); }
#else
  { return m_frame; }
#endif

public:
  PlotFileView(void);
  virtual ~PlotFileView(void);
  
  bool OnCreate(wxDocument *doc, long flags);
  void OnDraw(wxDC* dc);
  void OnUpdate(wxView *sender, wxObject *hint = NULL);
  bool OnClose (bool deleteWindow = true);

  void OnProperties (wxCommandEvent& event);
  void OnScaleMinMax (wxCommandEvent& event);
  void OnScaleAuto (wxCommandEvent& event);
  void OnScaleFull (wxCommandEvent& event);
  
  wxFrame* getFrame ()
  { return m_frame; }
  
  PlotFileDocument* GetDocument(void) 
  { return dynamic_cast<PlotFileDocument*>(wxView::GetDocument()); }
  
  DECLARE_EVENT_TABLE()
};

class PlotFileCanvas: public wxScrolledWindow
{
private:
  PlotFileView* m_pView;
  
public:
  PlotFileCanvas (PlotFileView* v, wxFrame *frame, const wxPoint& pos, const wxSize& size, const long style);
  virtual ~PlotFileCanvas()
  {}

  virtual void OnDraw(wxDC& dc);
};


#endif

